@@ -11,6 +11,7 @@ additional facilities.
 Command line options:
 
 --help        -- show usage info
+--offline     -- skip tests which require internet access
 -g;--generate -- write the output file for a test instead of comparing it.
                  The name of the test to write the output for must be
                  specified.
@@ -36,49 +37,89 @@ import traceback
 import unittest
 import doctest
 import distutils.util
+import gc
+
+def is_pypy():
+    import platform
+    try:
+        if platform.python_implementation()=='PyPy':
+            return True
+    except AttributeError:
+        #New in Python 2.6, not in Jython yet either
+        pass
+    return False
+
+def is_numpy():
+    if is_pypy():
+        return False
+    try:
+        import numpy
+        del numpy
+        return True
+    except ImportError:
+        return False
 
 # This is the list of modules containing docstring tests.
 # If you develop docstring tests for other modules, please add
-# those modules here.
-DOCTEST_MODULES = ["Bio.Application",
+# those modules here. Please sort names alphabetically.
+DOCTEST_MODULES = [
+                   "Bio.Align",
+                   "Bio.Align.Generic",
+                   "Bio.Align.Applications._Clustalw",
+                   "Bio.Align.Applications._ClustalOmega",
+                   "Bio.Align.Applications._Mafft",
+                   "Bio.Align.Applications._Muscle",
+                   "Bio.Align.Applications._Probcons",
+                   "Bio.Align.Applications._Prank",
+                   "Bio.Align.Applications._TCoffee",
+                   "Bio.AlignIO",
+                   "Bio.AlignIO.StockholmIO",
+                   "Bio.Alphabet",
+                   "Bio.Application",
+                   "Bio.Blast.Applications",
+                   "Bio.Emboss.Applications",
+                   "Bio.GenBank",
+                   "Bio.KEGG.Compound",
+                   "Bio.KEGG.Enzyme",
+                   "Bio.Motif",
+                   "Bio.pairwise2",
                    "Bio.Seq",
-                   "Bio.SeqFeature",
-                   "Bio.SeqRecord",
                    "Bio.SeqIO",
                    "Bio.SeqIO.AceIO",
                    "Bio.SeqIO.PhdIO",
                    "Bio.SeqIO.QualityIO",
                    "Bio.SeqIO.SffIO",
+                   "Bio.SeqFeature",
+                   "Bio.SeqRecord",
                    "Bio.SeqUtils",
-                   "Bio.Align",
-                   "Bio.Align.Generic",
-                   "Bio.AlignIO",
-                   "Bio.AlignIO.StockholmIO",
-                   "Bio.Blast.Applications",
-                   "Bio.Clustalw",
-                   "Bio.Emboss.Applications",
-                   "Bio.KEGG.Compound",
-                   "Bio.KEGG.Enzyme",
+                   "Bio.SeqUtils.MeltingTemp",
+                   "Bio.Sequencing.Applications._Novoalign",
                    "Bio.Wise",
                    "Bio.Wise.psw",
-                   "Bio.Motif",
                   ]
 #Silently ignore any doctests for modules requiring numpy!
-try:
-    import numpy
+if is_numpy():
     DOCTEST_MODULES.extend(["Bio.Statistics.lowess",
                             "Bio.PDB.Polypeptide",
+                            "Bio.PDB.Selection"
                             ])
-except ImportError:
-    pass
 
-#Skip Bio.Seq doctest under Python 3.0, see http://bugs.python.org/issue7490
-if sys.version_info[0:2] == (3,1):
+
+try:
+    import sqlite3
+    del sqlite3
+except ImportError:
+    #Missing on Jython or Python 2.4
+    DOCTEST_MODULES.remove("Bio.SeqIO")
+
+#Skip Bio.Seq doctest under Python 3, see http://bugs.python.org/issue7490
+if sys.version_info[0] == 3:
     DOCTEST_MODULES.remove("Bio.Seq")
 
 system_lang = os.environ.get('LANG', 'C') #Cache this
 
 def main(argv):
+    """Run tests, return number of failures (integer)."""
     # insert our paths in sys.path:
     # ../build/lib.*
     # ..
@@ -104,7 +145,7 @@ def main(argv):
     # get the command line options
     try:
         opts, args = getopt.getopt(argv, 'gv', ["generate", "verbose",
-            "doctest", "help"])
+            "doctest", "help", "offline"])
     except getopt.error, msg:
         print msg
         print __doc__
@@ -117,6 +158,12 @@ def main(argv):
         if o == "--help":
             print __doc__
             return 0
+        if o == "--offline":
+            print "Skipping any tests requiring internet access"
+            #This is a bit of a hack...
+            import requires_internet
+            requires_internet.check.available = False
+            #The check() function should now report internet not available
         if o == "-g" or o == "--generate":
             if len(args) > 1:
                 print "Only one argument (the test name) needed for generate"
@@ -143,9 +190,12 @@ def main(argv):
         if args[arg_num][-3:] == ".py":
             args[arg_num] = args[arg_num][:-3]
 
+    print "Python version:", sys.version
+    print "Operating system:", os.name, sys.platform
+
     # run the tests
     runner = TestRunner(args, verbosity)
-    runner.run()
+    return runner.run()
 
 
 class ComparisonTestCase(unittest.TestCase):
@@ -187,8 +237,10 @@ class ComparisonTestCase(unittest.TestCase):
         # the first line of the output file is the test name
         expected_test = expected.readline().strip()
 
-        assert expected_test == self.name, "\nOutput:   %s\nExpected: %s" % \
-               (self.name, expected_test)
+        if expected_test != self.name:
+            expected.close()
+            raise ValueError("\nOutput:   %s\nExpected: %s" \
+                  % (self.name, expected_test))
 
         # now loop through the output and compare it to the expected file
         while True:
@@ -212,10 +264,11 @@ class ComparisonTestCase(unittest.TestCase):
             if re.compile("^Ran [0-9]+ tests? in ").match(expected_line):
                 pass
             # otherwise make sure the two lines are the same
-            else:
-                assert expected_line == output_line, \
-                      "\nOutput  : %s\nExpected: %s" \
-                      % (repr(output_line), repr(expected_line))
+            elif expected_line != output_line:
+                expected.close()
+                raise ValueError("\nOutput  : %s\nExpected: %s" \
+                      % (repr(output_line), repr(expected_line)))
+        expected.close()
 
     def generate_output(self):
         """Generate the golden output for the specified test.
@@ -275,65 +328,78 @@ class TestRunner(unittest.TextTestRunner):
         # test changed this, e.g. to help with detecting command line tools)
         global system_lang
         os.environ['LANG']=system_lang
-        # Run the actual test inside a try/except to catch import errors.
-        # Have to do a nested try because try/except/except/finally requires
-        # python 2.5+
+        # Note the current directory:
+        cur_dir = os.path.abspath(".")
         try:
-            try:
-                stdout = sys.stdout
-                sys.stdout = output
-                if name.startswith("test_"):
-                    sys.stderr.write("%s ... " % name)
-                    #It's either a unittest or a print-and-compare test
-                    suite = unittest.TestLoader().loadTestsFromName(name)
-                    if suite.countTestCases()==0:
-                        # This is a print-and-compare test instead of a
-                        # unittest-type test.
-                        test = ComparisonTestCase(name, output)
-                        suite = unittest.TestSuite([test])
-                else:
-                    #It's a doc test
-                    sys.stderr.write("%s docstring test ... " % name)
-                    #Can't use fromlist=name.split(".") until python 2.5+
-                    module = __import__(name, None, None, name.split("."))
-                    suite = doctest.DocTestSuite(module)
-                    del module
-                suite.run(result)
-                if result.wasSuccessful():
-                    sys.stderr.write("ok\n")
-                    return True
-                else:
-                    sys.stderr.write("FAIL\n")
+            stdout = sys.stdout
+            sys.stdout = output
+            if name.startswith("test_"):
+                sys.stderr.write("%s ... " % name)
+                #It's either a unittest or a print-and-compare test
+                suite = unittest.TestLoader().loadTestsFromName(name)
+                if suite.countTestCases()==0:
+                    # This is a print-and-compare test instead of a
+                    # unittest-type test.
+                    test = ComparisonTestCase(name, output)
+                    suite = unittest.TestSuite([test])
+            else:
+                #It's a doc test
+                sys.stderr.write("%s docstring test ... " % name)
+                #Can't use fromlist=name.split(".") until python 2.5+
+                module = __import__(name, None, None, name.split("."))
+                suite = doctest.DocTestSuite(module)
+                del module
+            suite.run(result)
+            if cur_dir != os.path.abspath("."):
+                sys.stderr.write("FAIL\n")
+                result.stream.write(result.separator1+"\n")
+                result.stream.write("ERROR: %s\n" % name)
+                result.stream.write(result.separator2+"\n")
+                result.stream.write("Current directory changed\n")
+                result.stream.write("Was: %s\n" % cur_dir)
+                result.stream.write("Now: %s\n" % os.path.abspath("."))
+                os.chdir(cur_dir)
+                if not result.wasSuccessful():
                     result.printErrors()
                 return False
-            except MissingExternalDependencyError, msg:
-                sys.stderr.write("skipping. %s\n" % msg)
+            elif result.wasSuccessful():
+                sys.stderr.write("ok\n")
                 return True
-            except Exception, msg:
-                # This happened during the import
-                sys.stderr.write("ERROR\n")
-                result.stream.write(result.separator1+"\n")
-                result.stream.write("ERROR: %s\n" % name)
-                result.stream.write(result.separator2+"\n")
-                result.stream.write(traceback.format_exc())
-                return False
-            except KeyboardInterrupt, err:
-                # Want to allow this, and abort the test
-                # (see below for special case)
-                raise err
-            except:
-                # This happens in Jython with java.lang.ClassFormatError:
-                # Invalid method Code length ...
-                sys.stderr.write("ERROR\n")
-                result.stream.write(result.separator1+"\n")
-                result.stream.write("ERROR: %s\n" % name)
-                result.stream.write(result.separator2+"\n")
-                result.stream.write(traceback.format_exc())
-                return False
+            else:
+                sys.stderr.write("FAIL\n")
+                result.printErrors()
+            return False
+        except MissingExternalDependencyError, msg:
+            sys.stderr.write("skipping. %s\n" % msg)
+            return True
+        except Exception, msg:
+            # This happened during the import
+            sys.stderr.write("ERROR\n")
+            result.stream.write(result.separator1+"\n")
+            result.stream.write("ERROR: %s\n" % name)
+            result.stream.write(result.separator2+"\n")
+            result.stream.write(traceback.format_exc())
+            return False
+        except KeyboardInterrupt, err:
+            # Want to allow this, and abort the test
+            # (see below for special case)
+            raise err
+        except:
+            # This happens in Jython with java.lang.ClassFormatError:
+            # Invalid method Code length ...
+            sys.stderr.write("ERROR\n")
+            result.stream.write(result.separator1+"\n")
+            result.stream.write("ERROR: %s\n" % name)
+            result.stream.write(result.separator2+"\n")
+            result.stream.write(traceback.format_exc())
+            return False
         finally:
             sys.stdout = stdout
+            #Running under PyPy we were leaking file handles...
+            gc.collect()
 
     def run(self):
+        """Run tests, return number of failures (integer)."""
         failures = 0
         startTime = time.time()
         for test in self.tests:
@@ -350,8 +416,11 @@ class TestRunner(unittest.TextTestRunner):
         sys.stderr.write("\n")
         if failures:
             sys.stderr.write("FAILED (failures = %d)\n" % failures)
+        return failures
 
 
 if __name__ == "__main__":
-    #Don't do a sys.exit(...) as it isn't nice if run from IDLE.
-    main(sys.argv[1:])
+    errors = main(sys.argv[1:])
+    if errors:
+        #Doing a sys.exit(...) isn't nice if run from IDLE...
+        sys.exit(1)
